use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Andy
Benchmarks
Benct
David
Golden
Hash
IxHash
JIT
Jonsson
Lester
Mario
Ordered
Philip
Roy
andy
bpjonsson
concat
dagolden
darwin
dec
decrement
decrementing
incrementing
lib
marioeroy
postdec
postinc
predec
preinc
tombstoned
